@@ -42,11 +42,9 @@ pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd)
 		PDL_RETERROR(PDL_err, pdl_make_physvaffine(trans->pdls[j]));
 	}
 	for(; j<trans->vtable->npdls; j++) {
-		if(trans->pdls[j]->trans_parent != trans) {
-			if(VAFFINE_FLAG_OK(trans->vtable->per_pdl_flags,j))
-				par_pvaf++;
-			PDL_RETERROR(PDL_err, pdl_make_physvaffine(trans->pdls[j]));
-		}
+		if(VAFFINE_FLAG_OK(trans->vtable->per_pdl_flags,j))
+			par_pvaf++;
+		PDL_RETERROR(PDL_err, pdl_make_physvaffine(trans->pdls[j]));
 		flag |= trans->pdls[j]->state & PDL_ANYCHANGED;
 	}
 	if (flag & PDL_PARENTDIMSCHANGED) REDODIMS(trans);
@@ -78,7 +76,8 @@ pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd)
 }
 
 pdl *pdl_null() {
-	PDL_Anyval zero = { PDL_B, {0} };
+	PDL_Anyval zero = { PDL_D, {.D=0.0} };
+	PDLDEBUG_f(printf("pdl_null\n"));
 	pdl *it = pdl_pdlnew();
 	if (!it) return it;
 	pdl_error PDL_err = pdl_makescratchhash(it, zero);
@@ -87,6 +86,7 @@ pdl *pdl_null() {
 }
 
 pdl *pdl_scalar(PDL_Anyval anyval) {
+	PDLDEBUG_f(printf("pdl_scalar type=%d Bval=%d\n", anyval.type, (int)anyval.value.B));
 	pdl *it = pdl_pdlnew();
 	if (!it) return it;
 	pdl_error PDL_err = pdl_makescratchhash(it, anyval);
@@ -99,6 +99,7 @@ pdl *pdl_scalar(PDL_Anyval anyval) {
 
 pdl *pdl_get_convertedpdl(pdl *old,int type) {
 	if(old->datatype == type) return old;
+	PDLDEBUG_f(printf("pdl_get_convertedpdl\n"));
 	pdl *it = pdl_null();
 	if (!it) return it;
 	pdl_error PDL_err = pdl_converttypei_new(old,it,type);
@@ -128,7 +129,6 @@ pdl_error pdl_grow(pdl* a, PDL_Indx newsize) {
      SV *sv = get_sv("PDL::BIGPDL",0);
      if(sv == NULL || !(SvTRUE(sv)))
        return pdl_make_error_simple(PDL_EUSERERROR, "Probably false alloc of over 1Gb PDL! (set $PDL::BIGPDL = 1 to enable)");
-     fflush(stdout);
    }
    (void)SvGROW ( foo, nbytes );
    SvCUR_set( foo, nbytes );
@@ -159,7 +159,7 @@ pdl* pdl_pdlnew() {
      if (!it) return it;
      memset(it, 0, sizeof(pdl));
      it->magicno = PDL_MAGICNO;
-     it->datatype = 0;
+     it->datatype = PDL_D;
      it->trans_parent = NULL;
      it->vafftrans = NULL;
      it->sv = NULL;
@@ -235,8 +235,7 @@ pdl_error pdl__free(pdl *it) {
     PDL_CHKMAGIC(it);
     /* now check if magic is still there */
     if (pdl__ismagic(it)) {
-      PDLDEBUG_f(printf("%p is still magic\n",(void*)it));
-      PDLDEBUG_f(pdl__print_magic(it));
+      PDLDEBUG_f(printf("%p is still magic\n",(void*)it);pdl__print_magic(it));
     }
     it->magicno = 0x42424245;
     if(it->dims       != it->def_dims)       free((void*)it->dims);
@@ -271,27 +270,18 @@ pdl_error pdl__free(pdl *it) {
     return PDL_err;
 }
 
-/* Problem with this function: when transformation is destroyed,
- * there may be several different children with the same name.
- * Therefore, we cannot croak :(
- */
-void pdl__removechildtrans(pdl *it,pdl_trans *trans, PDL_Indx nth,int all)
+void pdl__removechildtrans(pdl *it,pdl_trans *trans)
 {
-	PDLDEBUG_f(printf("pdl__removechildtrans(%p): %p %"IND_FLAG"\n",
-	  (void*)trans, (void*)(it), nth));
+	PDLDEBUG_f(printf("pdl__removechildtrans(%s=%p): %p\n",
+	  trans->vtable->name, (void*)trans, (void*)(it)));
 	PDL_Indx i; int flag = 0;
-	if(all) {
-		for(i=0; i<trans->vtable->nparents; i++)
-			if(trans->pdls[i] == it)
-				trans->pdls[i] = NULL;
-	} else {
-		trans->pdls[nth] = 0;
-	}
+	for(i=0; i<trans->vtable->nparents; i++)
+		if(trans->pdls[i] == it)
+			trans->pdls[i] = NULL;
 	PDL_DECL_CHILDLOOP(it);
 	PDL_START_CHILDLOOP(it)
 		if (PDL_CHILDLOOP_THISCHILD(it) != trans) continue;
 		PDL_CHILDLOOP_THISCHILD(it) = NULL;
-		if(!all) return;
 		flag = 1;
 		/* Can't return; might be many times (e.g. $x+$x) */
 	PDL_END_CHILDLOOP(it)
@@ -303,8 +293,8 @@ void pdl__removechildtrans(pdl *it,pdl_trans *trans, PDL_Indx nth,int all)
 
 void pdl__removeparenttrans(pdl *it, pdl_trans *trans, PDL_Indx nth)
 {
-	PDLDEBUG_f(printf("pdl__removeparenttrans(%p): %p %"IND_FLAG"\n",
-	  (void*)trans, (void*)(it), nth));
+	PDLDEBUG_f(printf("pdl__removeparenttrans(%s=%p): %p %"IND_FLAG"\n",
+	  trans->vtable->name, (void*)trans, (void*)(it), nth));
 	trans->pdls[nth] = 0;
 	it->trans_parent = 0;
 }
@@ -315,12 +305,13 @@ pdl_error pdl_destroytransform(pdl_trans *trans,int ensure,int *wd)
 	PDL_TR_CHKMAGIC(trans);
 	PDL_Indx j;
 	int ismutual = (trans->flags & PDL_ITRANS_DO_DATAFLOW_ANY);
+	if (!trans->vtable)
+		return pdl_make_error(PDL_EFATAL, "ZERO VTABLE DESTTRAN 0x%p %d\n",trans,ensure);
 	if (!ismutual) for(j=0; j<trans->vtable->nparents; j++)
 	  if (trans->pdls[j]->state & PDL_DATAFLOW_ANY) { ismutual=1; break; }
-	PDLDEBUG_f(printf("pdl_destroytransform %p (ensure %d, ismutual %d)\n",
+	PDLDEBUG_f(printf("pdl_destroytransform %s=%p (ensure=%d ismutual=%d)\n",
+			  trans->vtable ? trans->vtable->name : "NULL",
 			  (void*)trans,ensure,ismutual));
-	if(!trans->vtable)
-		return pdl_make_error(PDL_EFATAL, "ZERO VTABLE DESTTRAN 0x%p %d\n",trans,ensure);
 	if(ensure)
 		PDL_RETERROR(PDL_err, pdl__ensure_trans(trans,ismutual ? 0 : PDL_PARENTDIMSCHANGED,wd));
 	pdl *destbuffer[trans->vtable->npdls];
@@ -330,7 +321,7 @@ pdl_error pdl_destroytransform(pdl_trans *trans,int ensure,int *wd)
 	    pdl *parent = trans->pdls[j];
 	    if(!parent) continue;
 	    PDL_CHKMAGIC(parent);
-	    pdl__removechildtrans(parent,trans,j,1);
+	    pdl__removechildtrans(parent,trans);
 	    if(!(parent->state & PDL_DESTROYING) && !parent->sv)
 	      destbuffer[ndest++] = parent;
 	  }
@@ -384,9 +375,9 @@ pdl_error pdl_destroy(pdl *it) {
     int nafn=0;
     PDL_DECL_CHILDLOOP(it);
     PDL_CHKMAGIC(it);
-    PDLDEBUG_f(printf("pdl_destroy %p\n",(void*)it);)
+    PDLDEBUG_f(printf("pdl_destroy %p\n",(void*)it));
     if(it->state & PDL_DESTROYING) {
-        PDLDEBUG_f(printf("  already destroying, returning\n");)
+        PDLDEBUG_f(printf("  already destroying, returning\n"));
 	return PDL_err;
     }
     it->state |= PDL_DESTROYING;
@@ -427,7 +418,7 @@ pdl_error pdl_destroy(pdl *it) {
  */
     if(nafn) goto soft_destroy;
     if(pdl__magic_isundestroyable(it)) {
-        PDLDEBUG_f(printf("pdl_destroy not destroying as magic %p\n",(void*)it);)
+        PDLDEBUG_f(printf("pdl_destroy not destroying as magic %p\n",(void*)it));
 	goto soft_destroy;
     }
 
@@ -444,12 +435,12 @@ pdl_error pdl_destroy(pdl *it) {
 /* Here, this is a child but has no children - fall through to hard_destroy */
 
    PDL_RETERROR(PDL_err, pdl__free(it));
-   PDLDEBUG_f(printf("pdl_destroy end %p\n",(void*)it);)
+   PDLDEBUG_f(printf("pdl_destroy end %p\n",(void*)it));
    return PDL_err;
 
   soft_destroy:
     PDLDEBUG_f(printf("pdl_destroy may have dependencies, not destroy %p, nba(%d, %d), nforw(%d), tra(%p), nafn(%d)\n",
-				(void*)it, nback, nback2, nforw, (void*)(it->trans_parent), nafn);)
+				(void*)it, nback, nback2, nforw, (void*)(it->trans_parent), nafn));
     it->state &= ~PDL_DESTROYING;
     return PDL_err;
 }
@@ -460,10 +451,11 @@ pdl *pdl_hard_copy(pdl *src) {
 	pdl_error PDL_err = pdl_make_physical(src); /* Wasteful XXX... should be lazier */
 	if (PDL_err.error) return NULL;
 	int i;
+	PDLDEBUG_f(printf("pdl_hard_copy\n"));
 	pdl *it = pdl_null();
 	if (!it) return it;
 	it->state = 0;
-	PDLDEBUG_f(printf("pdl_hard_copy (%p): ", src));PDLDEBUG_f(pdl_dump(it);)
+	PDLDEBUG_f(printf("pdl_hard_copy (%p): ", src);pdl_dump(it));
 	it->datatype = src->datatype;
 	PDL_err = pdl_setdims(it,src->dims,src->ndims);
 	if (PDL_err.error) { pdl_destroy(it); return NULL; }
@@ -546,7 +538,7 @@ void pdl_resize_defaultincs(pdl *it) {
 /* Init dims & incs - if *incs is NULL ignored (but space is always same for both)  */
 pdl_error pdl_setdims(pdl* it, PDL_Indx * dims, PDL_Indx ndims) {
   pdl_error PDL_err = {0, NULL, 0};
-  PDLDEBUG_f(printf("pdl_setdims %p: ", it));PDLDEBUG_f(pdl_print_iarr(dims, ndims));PDLDEBUG_f(printf("\n"));
+  PDLDEBUG_f(printf("pdl_setdims %p: ", it);pdl_print_iarr(dims, ndims);printf("\n"));
   PDL_Indx i, old_nvals = it->nvals, new_nvals = 1;
   for (i=0; i<ndims; i++) new_nvals *= dims[i];
   int what = (old_nvals == new_nvals) ? 0 : PDL_PARENTDATACHANGED;
@@ -556,7 +548,6 @@ pdl_error pdl_setdims(pdl* it, PDL_Indx * dims, PDL_Indx ndims) {
     for (i=0; i<ndims; i++)
       if (dims[i] != it->dims[i]) { what |= PDL_PARENTDIMSCHANGED; break; }
   if (!what) { PDLDEBUG_f(printf("pdl_setdims NOOP\n")); return PDL_err; }
-  PDL_RETERROR(PDL_err, pdl_changesoon(it));
   PDL_RETERROR(PDL_err, pdl_reallocdims(it,ndims));
   for(i=0; i<ndims; i++) it->dims[i] = dims[i];
   pdl_resize_defaultincs(it);
@@ -615,8 +606,8 @@ pdl_error pdl_make_physdims(pdl *it) {
 	  return PDL_err;
 	}
 	it->state &= ~PDL_PARENTDIMSCHANGED;
-	/* the fact that a PARENTXXXCHANGED flag is set seems
-	   to imply that this pdl has an associated trans ? */
+	PDLDEBUG_f(printf("make_physdims %p TRANS:\n",(void*)it);
+	    pdl_dump_trans_fixspace(it->trans_parent,3));
 	for(i=0; i<it->trans_parent->vtable->nparents; i++) {
 		PDL_RETERROR(PDL_err, pdl_make_physdims(it->trans_parent->pdls[i]));
 	}
@@ -665,8 +656,7 @@ static inline pdl_error pdl_trans_flow_checks(pdl_trans *trans, int *ret) {
 pdl_error pdl_make_trans_mutual(pdl_trans *trans)
 {
   pdl_error PDL_err = {0, NULL, 0};
-  PDLDEBUG_f(printf("make_trans_mutual %p\n",(void*)trans));
-  PDLDEBUG_f(pdl_dump_trans_fixspace(trans,3));
+  PDLDEBUG_f(printf("make_trans_mutual %p\n",(void*)trans);pdl_dump_trans_fixspace(trans,3));
   pdl_transvtable *vtable = trans->vtable;
   PDL_Indx i, npdls=vtable->npdls, nparents=vtable->nparents;
   PDL_TR_CHKMAGIC(trans);
@@ -689,8 +679,7 @@ pdl_error pdl_make_trans_mutual(pdl_trans *trans)
 		   previous parent transformations and mutate if they exist
 		   if no dataflow. */
 		child->state |= PDL_PARENTDIMSCHANGED | PDL_PARENTDATACHANGED;
-	} else
-		PDL_RETERROR(PDL_err, pdl_changesoon(child));
+	}
 	if (dataflow || isnull) child->trans_parent = trans;
 	if (isnull)
 	    child->state = (child->state & ~PDL_NOMYDIMS) | PDL_MYDIMS_TRANS;
@@ -703,8 +692,7 @@ pdl_error pdl_make_trans_mutual(pdl_trans *trans)
 
 pdl_error pdl_redodims_default(pdl_trans *trans) {
   pdl_error PDL_err = {0, NULL, 0};
-  PDLDEBUG_f(printf("pdl_redodims_default "));
-  PDLDEBUG_f(pdl_dump_trans_fixspace(trans,0));
+  PDLDEBUG_f(printf("pdl_redodims_default ");pdl_dump_trans_fixspace(trans,0));
   pdl_transvtable *vtable = trans->vtable;
   PDL_Indx creating[vtable->npdls];
   pdl **pdls = trans->pdls;
@@ -779,54 +767,16 @@ pdl_error pdl_make_physical(pdl *it) {
 	return PDL_err;
 }
 
-pdl_error pdl_children_changesoon_c(pdl *it)
-{
-	pdl_error PDL_err = {0, NULL, 0};
-	int i;
-	PDL_DECL_CHILDLOOP(it);
-	PDL_START_CHILDLOOP(it)
-	    pdl_trans *t = PDL_CHILDLOOP_THISCHILD(it);
-	    if (t->flags & PDL_ITRANS_DO_DATAFLOW_F)
-		for(i=t->vtable->nparents; i<t->vtable->npdls; i++)
-		    PDL_RETERROR(PDL_err, pdl_children_changesoon_c(t->pdls[i]));
-	    else
-		PDL_RETERROR(PDL_err, pdl_destroytransform(t,1,NULL));
-	PDL_END_CHILDLOOP(it)
-	return PDL_err;
-}
-
-/* Change soon: if this is not writeback, separate from
-   parent.
-   If the children of this are not writeback, separate them.
- */
-pdl_error pdl_changesoon(pdl *it)
-{
-    pdl_error PDL_err = {0, NULL, 0};
-    unsigned int i;
-    if (it->trans_parent) {
-	if (it->trans_parent->flags & PDL_ITRANS_DO_DATAFLOW_B) {
-	    if(!(it->trans_parent->flags & PDL_ITRANS_TWOWAY))
-		return pdl_make_error_simple(PDL_EUSERERROR, "PDL: Internal error: Trying to writeback non-TWOWAY trans");
-	    for(i=0; i<it->trans_parent->vtable->nparents; i++)
-		PDL_RETERROR(PDL_err, pdl_changesoon(it->trans_parent->pdls[i]));
-	    return PDL_err;
-	} else
-	    PDL_RETERROR(PDL_err, pdl_destroytransform(it->trans_parent,1,NULL));
-    }
-    PDL_RETERROR(PDL_err, pdl_children_changesoon_c(it));
-    return PDL_err;
-}
-
 pdl_error pdl_changed(pdl *it, int what, int recursing)
 {
     pdl_error PDL_err = {0, NULL, 0};
     int i; int j;
-    PDLDEBUG_f(do {
+    PDLDEBUG_f(
       printf("pdl_changed: entry for pdl %p recursing: %d, what ",
 	     (void*)it,recursing);
       pdl_dump_flags_fixspace(what,0,PDL_FLAGS_PDL);
       if (it->state & PDL_TRACEDEBUG) pdl_dump(it);
-    } while (0));
+    );
     if(recursing) {
 	it->state |= what;
 	if(pdl__ismagic(it)) pdl__call_magic(it,PDL_MAGIC_MARKCHANGED);
@@ -1008,7 +958,10 @@ pdl_error pdl_set_datatype(pdl *a, int datatype)
     PDL_RETERROR(PDL_err, pdl_make_physical(a));
     if(a->trans_parent)
 	PDL_RETERROR(PDL_err, pdl_destroytransform(a->trans_parent,1,NULL));
-    PDL_RETERROR(PDL_err, pdl_converttype( a, datatype ));
+    if (a->state & PDL_NOMYDIMS)
+	a->datatype = datatype;
+    else
+	PDL_RETERROR(PDL_err, pdl_converttype( a, datatype ));
     return PDL_err;
 }
 
@@ -1064,10 +1017,10 @@ void pdl_propagate_badvalue( pdl *it ) {
 } /* pdl_propagate_badvalue */
 
 PDL_Anyval pdl_get_badvalue( int datatype ) {
-    PDL_Anyval retval = { -1, {0} };
+    PDL_Anyval retval = { PDL_INVALID, {0} };
 #define X(datatype, ctype, ppsym, shortctype, defbval) \
     retval.type = datatype; retval.value.ppsym = PDL.bvals.shortctype;
-    PDL_GENERICSWITCH(datatype, X, return retval)
+    PDL_GENERICSWITCH(PDL_TYPELIST2_ALL, datatype, X, return retval)
 #undef X
     return retval;
 }
@@ -1100,6 +1053,8 @@ pdl_trans *pdl_create_trans(pdl_transvtable *vtable) {
     if (!it->inc_sizes) return NULL;
     for (i=0; i<vtable->nind_ids; i++) it->inc_sizes[i] = -1;
     it->offs = -1;
+    it->incs = NULL;
+    it->__datatype = PDL_INVALID;
     return it;
 }
 
@@ -1109,11 +1064,13 @@ pdl_error pdl_type_coerce(pdl_trans *trans) {
   pdl_transvtable *vtable = trans->vtable;
   pdl **pdls = trans->pdls;
   trans->__datatype = -1;
+  char parent_has_badvalue = 0;
+  PDL_Anyval parent_badvalue = {PDL_INVALID, {0}};
   if (vtable->npdls == 2 && pdls[0]->has_badvalue
       && (vtable->par_flags[1] & PDL_PARAM_ISCREATEALWAYS)) {
     /* P2Child case */
-    trans->has_badvalue = 1;
-    trans->badvalue = pdls[0]->badvalue;
+    parent_has_badvalue = 1;
+    parent_badvalue = pdls[0]->badvalue;
   }
   for (i=0; i<vtable->npdls; i++) {
     pdl *pdl = pdls[i];
@@ -1148,12 +1105,12 @@ pdl_error pdl_type_coerce(pdl_trans *trans) {
     } else if (flags & PDL_PARAM_ISCOMPLEX) {
       if (trans_dtype < PDL_CF) new_dtype = PDLMAX(PDL_CF, trans_dtype + (PDL_CF - PDL_F));
     }
-    if ((flags & PDL_PARAM_ISCREATEALWAYS) ||
-       ((flags & PDL_PARAM_ISCREAT) && (pdl->state & PDL_NOMYDIMS) && pdl->trans_parent == NULL)) {
-      pdl->badvalue = trans->badvalue;
-      pdl->has_badvalue = trans->has_badvalue;
+    if ((pdl->state & PDL_NOMYDIMS) && (!pdl->trans_parent || pdl->trans_parent == trans)) {
+      pdl->badvalue = parent_badvalue;
+      pdl->has_badvalue = parent_has_badvalue;
       pdl->datatype = new_dtype;
     } else if (new_dtype != pdl->datatype) {
+      PDLDEBUG_f(printf("pdl_type_coerce (%s) pdl=%"IND_FLAG" from %d to %d\n", vtable->name, i, pdl->datatype, new_dtype));
       pdl = pdl_get_convertedpdl(pdl, new_dtype);
       if (!pdl)
         return pdl_make_error(PDL_EFATAL, "%s got NULL pointer from get_convertedpdl on param %s", vtable->name, vtable->par_names[i]);
