@@ -1,24 +1,17 @@
-#
-#
 # ToDo:
 #  - multiple windows - requires editing generate.pl in OpenGL/
 #  - clean up
 #
 #package PDL::Graphics::TriD::GL;
 
+use strict;
+use warnings;
+use OpenGL qw(:all);
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 
-BEGIN {
-   use PDL::Config;
-   if ( $PDL::Config{USE_POGL} ) {
-      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
-      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
-   } else {
-      eval 'use PDL::Graphics::OpenGL';
-   }
-}
-
+$PDL::Graphics::TriD::create_window_sub = # warnings
 $PDL::Graphics::TriD::create_window_sub = sub {
-  return new PDL::Graphics::TriD::GL::Window(@_);
+  return PDL::Graphics::TriD::GL::Window->new(@_);
 };
 
 sub PDL::Graphics::TriD::Material::togl{
@@ -143,7 +136,7 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
    
         my (@nadd,@nc,@ns);
 
-	for $dim (0..1) {
+	for my $dim (0..1) {
 	  my $width = $this->{Scale}[$dim][1]-$this->{Scale}[$dim][0];
 	  if($width > 100){
 	    $nadd[$dim] = 10;
@@ -161,7 +154,7 @@ sub PDL::Graphics::TriD::CylindricalEquidistantAxes::togl_axis {
 	# can be changed to topo heights?
 	my $verts = zeroes(3,$ns[0],$ns[1]);
 
-	($t = $verts->slice("2")) .= 1012.5;
+	(my $t = $verts->slice("2")) .= 1012.5;
 	($t = $verts->slice("0")) .= $verts->ylinvals($nc[0],$nc[0]+$nadd[0]*($ns[0]-1));
 	($t = $verts->slice("1")) .= $verts->zlinvals($nc[1],$nc[1]+$nadd[1]*($ns[1]-1));
 
@@ -232,15 +225,11 @@ sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 		my $nc = $s->[0];
 		for(0..$ndiv) {
 			&glRasterPos3f(@coords);
-                        if ( $PDL::Config{USE_POGL} ) {
-                              if ( OpenGL::done_glutInit() ) {
-                                 OpenGL::glutBitmapString($fontbase, sprintf("%.3f",$nc));
-                              } else {
-                                 OpenGL::glpPrintString($fontbase, sprintf("%.3f",$nc));
-                              }
-                           } else {
-                              PDL::Graphics::OpenGL::glpPrintString($fontbase, sprintf("%.3f",$nc));
-                           }
+			if ( OpenGL::done_glutInit() ) {
+			   OpenGL::glutBitmapString($fontbase, sprintf("%.3f",$nc));
+			} else {
+			   OpenGL::glpPrintString($fontbase, sprintf("%.3f",$nc));
+			}
 			glBegin(GL_LINES);
 			&glVertex3f(@coords0);
 			&glVertex3f(@coords);
@@ -252,15 +241,11 @@ sub PDL::Graphics::TriD::EuclidAxes::togl_axis {
 		}
 		$coords0[$dim] = 1.1;
 		&glRasterPos3f(@coords0);
-                if ( $PDL::Config{USE_POGL} ) {
-                   if ( OpenGL::done_glutInit() ) {
-                      OpenGL::glutBitmapString($fontbase, $this->{Names}[$dim]);
-                   } else {
-                      OpenGL::glpPrintString($fontbase, $this->{Names}[$dim]);
-                   }
-                } else {
-                   PDL::Graphics::OpenGL::glpPrintString($fontbase, $this->{Names}[$dim]);
-                }
+		if ( OpenGL::done_glutInit() ) {
+			OpenGL::glutBitmapString($fontbase, $this->{Names}[$dim]);
+		} else {
+			OpenGL::glpPrintString($fontbase, $this->{Names}[$dim]);
+		}
              }
 	glEnable(GL_LIGHTING);
 }
@@ -528,11 +513,11 @@ sub PDL::Graphics::TriD::SLattice_S::gdraw {
 ################### JNK 15mar11 added section start ################
 sub PDL::Graphics::TriD::STrigrid_S::gdraw {
   my($this,$points) = @_;
-  glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
+  glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
   # For some reason, we need to set this here as well.
-  glLightModeli(&GL_LIGHT_MODEL_TWO_SIDE, &GL_TRUE);
+  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
   # By-vertex doesn't make sense otherwise.
-  glShadeModel (&GL_SMOOTH);   
+  glShadeModel (GL_SMOOTH);   
   my @sls = (":,(0)",":,(1)",":,(2)");
   my $idx = [0,1,2,0]; # for lines, below
   if ($this->{Options}{Smooth}) {
@@ -550,17 +535,17 @@ sub PDL::Graphics::TriD::STrigrid_S::gdraw {
                                       :\&PDL::gl_triangles_n_mat);
     &$f( (map {$this->{Faces}->slice($_)} @sls),   # faces is a slice of points
          (map {$this->{Colors}->slice($_)} @sls) ); } 
-  glDisable(&GL_LIGHTING);
+  glDisable(GL_LIGHTING);
   if ($this->{Options}{Lines}) {
     my $black = PDL->pdl(0,0,0)->dummy(1)->dummy(1);
     PDL::gl_lines($this->{Faces}->dice_axis(1,$idx),$black); } glPopAttrib(); }
 
 sub PDL::Graphics::TriD::STrigrid::gdraw {
   my($this,$points) = @_;
-  glPushAttrib(&GL_LIGHTING_BIT | &GL_ENABLE_BIT);
-  glDisable(&GL_LIGHTING);
+  glPushAttrib(GL_LIGHTING_BIT | GL_ENABLE_BIT);
+  glDisable(GL_LIGHTING);
 # By-vertex doesn't make sense otherwise.
-  glShadeModel (&GL_SMOOTH);
+  glShadeModel (GL_SMOOTH);
   my @sls = (":,(0)",":,(1)",":,(2)");
   my $idx = [0,1,2,0];
   PDL::gl_triangles(
@@ -602,11 +587,7 @@ sub PDL::Graphics::TriD::Image::gdraw {
 	my($this,$vert) = @_;
 	my ($p,$xd,$yd,$txd,$tyd) = $this->flatten(1); # do binary alignment
 	glColor3d(1,1,1);
-        if ( $PDL::Config{USE_POGL} ) {
-           glTexImage2D_s(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
-        } else {
-           glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
-        }
+         glTexImage2D_s(GL_TEXTURE_2D, 0, GL_RGB, $txd, $tyd, 0, GL_RGB, GL_FLOAT, $p->get_dataref());
 	 glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
 	    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
 	       glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
@@ -650,15 +631,8 @@ sub PDL::Graphics::TriD::SimpleController::togl {
 #
 package PDL::Graphics::TriD::Window;
 
-BEGIN {
-   use PDL::Config;
-   if ( $PDL::Config{USE_POGL} ) {
-      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
-      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
-   } else {
-      eval 'use PDL::Graphics::OpenGL';
-   }
-}
+use OpenGL qw(:all);
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 
 use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/Ev Width Height Interactive _GLObject 
@@ -746,11 +720,7 @@ sub gdriver {
 #  $this->reshape($options->{width},$options->{height});
 
   my $light = pack "f*",1.0,1.0,1.0,0.0;
-  if ( $PDL::Config{USE_POGL} ) {
-     glLightfv_s(GL_LIGHT0,GL_POSITION,$light);
-  } else {
-     glLightfv(GL_LIGHT0,GL_POSITION,$light);
-  }
+  glLightfv_s(GL_LIGHT0,GL_POSITION,$light);
 
   glColor3f(1,1,1);
   
@@ -812,7 +782,7 @@ sub twiddle {
          }
   }
   if(!defined $getout) {
-	 $getout = not $PDL::Graphics::TriD::keeptwiddling;
+	 $getout = not ($PDL::Graphics::TriD::keeptwiddling && $PDL::Graphics::TriD::keeptwiddling);
   }
   
   $this->display();
@@ -932,19 +902,13 @@ sub display {
 
   }
 
-  if ( $PDL::Config{USE_POGL} ) {
-
-     print "display: SwapBuffers() call on return\n" if ($PDL::Graphics::TriD::verbose);
-     if ( $this->{_GLObject}->{window_type} eq 'glut' ) {  # need to make method call
-        glutSwapBuffers();
-     } elsif ( $this->{_GLObject}->{window_type} eq 'x11' ) {  # need to make method call
-        $this->{_GLObject}->glXSwapBuffers();
-     } else {
-        print "display: got object with inconsistent _GLObject info\n";
-     }
-
+  print "display: SwapBuffers() call on return\n" if ($PDL::Graphics::TriD::verbose);
+  if ( $this->{_GLObject}->{window_type} eq 'glut' ) {  # need to make method call
+    glutSwapBuffers();
+  } elsif ( $this->{_GLObject}->{window_type} eq 'x11' ) {  # need to make method call
+    $this->{_GLObject}->glXSwapBuffers();
   } else {
-     $this->{_GLObject}->glXSwapBuffers();
+    print "display: got object with inconsistent _GLObject info\n";
   }
 #  $this->{Angle}+= 3;
 }
@@ -956,13 +920,7 @@ sub read_picture {
 	my $res = PDL->zeroes(PDL::byte,3,$w,$h);
 	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
 	glPixelStorei(GL_PACK_ALIGNMENT,1);
-
-        if ( $PDL::Config{USE_POGL} ) {
-           glReadPixels_s(0,0,$w,$h,GL_RGB,GL_UNSIGNED_BYTE,$res->get_dataref);
-        } else {
-           glReadPixels(0,0,$w,$h,GL_RGB,GL_UNSIGNED_BYTE,$res->get_dataref);
-        }
-
+        glReadPixels_s(0,0,$w,$h,GL_RGB,GL_UNSIGNED_BYTE,$res->get_dataref);
 	return $res;
 }
 
@@ -972,15 +930,8 @@ sub read_picture {
 
 package PDL::Graphics::TriD::EventHandler;
 
-BEGIN {
-   use PDL::Config;
-   if ( $PDL::Config{USE_POGL} ) {
-      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(ConfigureNotify MotionNotify ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask)";
-      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
-   } else {
-      eval 'use PDL::Graphics::OpenGL';
-   }
-}
+use OpenGL qw(ConfigureNotify MotionNotify ButtonPress ButtonRelease Button1Mask Button2Mask Button3Mask);
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 
 use fields qw/X Y Buttons VP/;
 use strict;
@@ -1067,19 +1018,10 @@ use base qw/PDL::Graphics::TriD::Object/;
 use fields qw/X0 Y0 W H Transformer EHandler Active ResizeCommands 
               DefMaterial AspectRatio Graphs/;
 
-BEGIN {
-   use PDL::Config;
-   if ( $PDL::Config{USE_POGL} ) {
-      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
-      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
-   } else {
-      eval 'use PDL::Graphics::OpenGL';
-   }
-}
-
+use OpenGL qw(:all);
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 use PDL::Graphics::OpenGLQ;
 
-
 sub highlight {
   my ($vp) = @_;
 
