@@ -7,22 +7,15 @@
 
 package PDL::Graphics::TriD::GoBoard;
 
+use strict;
+use warnings;
 use base qw/PDL::Graphics::TriD::Object/;
 use fields qw /Data InLays BG/;
 
-BEGIN {
-   use PDL::Config;
-   if ( $PDL::Config{USE_POGL} ) {
-      eval "use OpenGL $PDL::Config{POGL_VERSION} qw(:all)";
-      eval 'use PDL::Graphics::OpenGL::Perl::OpenGL';
-   } else {
-      eval 'use PDL::Graphics::OpenGL';
-   }
-}
-
+use OpenGL qw(:all);
+use PDL::Graphics::OpenGL::Perl::OpenGL;
 use PDL::Lite;
 
-
 sub new {
 	my($type,$opts) = @_;
 	my $this = $type->SUPER::new();
